meta:
  id: ax25frames
  endian: be
  
doc: |
  : MTCUBE - CELESTA telemetry description file 
  : Version 0.2
  :
  : reference documentation 
  :      MTCUBE - CELESTA  description for web site 
  :  
  : This ksy file is used for different automatic code generation 
  :    - ksy file to programming language thanks to ksy compiler 
       - ksy file to graphwiz document thnaks to ksy compiler 
       - Ksy file to java textual display files thanks to JOSAST GeneratePOJOfromKSY tool (experimental)
       - Ksy file to FXML files and associated java files file thanks to JOSAST GeneratePOJOfromKSY tool (experimental)
       
  : Writing rules should be respected (see JOSAST project)
  :   - for interpreted value, variable should finish with 
  :      _raw if no interpretation is done 
  :      _c, _ma _v _dbm _mw according to the interpreted value 
  :   - the name of the interpreted value should be close to the displayed name (to limit rework)
  :      _c, _ma _v _dbm _mw are automaticaly converted to (C) (mA) (dbm) (mW)


seq:
  - id: ax25_frame
    type: ax25_frame
    doc-ref: 'https://www.tapr.org/pub_ax25.html'

types:
  ax25_frame:
    seq:
    - id: ax25_header
      type: ax25_header
    - id: payload
      type:
        switch-on: ax25_header.ctl & 0x13
        cases:
          0x03: ui_frame
          0x13: ui_frame
          0x00: i_frame
          0x02: i_frame
          0x10: i_frame
          0x12: i_frame
          #0x11: s_frame

  ax25_header:
    seq:
      - id: dest_callsign_raw
        type: dest_callsign_raw
      - id: dest_ssid_raw
        type: u1
      - id: src_callsign_raw
        type: src_callsign_raw
      - id: src_ssid_raw
        type: u1
      - id: ctl
        type: u1
    instances:
      src_ssid:
        value: (src_ssid_raw & 0x0f) >> 1
      dest_ssid:
        value: (dest_ssid_raw & 0x0f) >> 1
  dest_callsign_raw:
    seq:
      - id: dest_callsign_ror
        process: ror(1)
        size: 6
        type: dest_callsign
  src_callsign_raw:
    seq:
      - id: src_callsign_ror
        process: ror(1)
        type: src_callsign
        size: 6
  dest_callsign:
    seq:
      - id: dest_callsign
        type: str
        encoding: ASCII
        size: 6
  src_callsign:
    seq:
      - id: src_callsign
        type: str
        encoding: ASCII
        size: 6

  i_frame:
    seq:
      - id: pid
        type: u1
      - id: ax25_info
        size-eos: true

  ui_frame:
    seq:
      - id: pid
        type: u1
      - id: size
        type: u1
      - id: frame_type
        type: u1
      - id: ts
        type: u4
      - id: obdh
        type: obdh
      - id: eps
        type: eps
      - id: ttc
        type: ttc
      - id: payload
        size: 48 
      - id: ham_message
        type: ham_message
      - id: ax25_info
        size-eos: true
        
        
  obdh:
    seq:
      - id: timestamp
        type: u4
      - id: temperature
        type: s2
      - id: satellite_mode
        type: u1
      - id: obdh_mode
        type: u1
      - id: bytes_to_transmit
        type: u4
      - id: number_of_resets
        type: u2
      - id: number_of_errors
        type: u2
    instances:
     obdh_timestamp_raw:
      value:  timestamp
     obdh_temperature_c:
      value:  temperature
     obdh_satellite_mode_raw:
      value:  satellite_mode
     obdh_obdh_mode_raw:
      value:  obdh_mode
     obdh_bytes_to_transmit_raw:
      value:  bytes_to_transmit
     obdh_number_of_resets_raw:
      value:  number_of_resets
     obdh_number_of_errors_raw:
      value:  number_of_errors    
      

  eps:
    seq:
      - id: eps_mode
        type: u1
      - id: battery_voltage
        type: u1
      - id: battery_temperature
        type: s1
      - id: min_battery_voltage
        type: u1
      - id: max_battery_voltage
        type: u1
      - id: avg_battery_voltage
        type: u1
      - id: avg_charge_current
        type: u1 
      - id: max_charge_current
        type: u1
      - id: z_minu_face_temperature
        type: s1
      - id: o_b_d_h_current
        type: u1
      - id: e_p_s_current
        type: u1
      - id: t_t_c_micro_c_current
        type: u1
      - id: t_t_c_p_a_current
        type: u1
    #Current consumption of the Radiation Dosimeter Instrument
      - id: d_o_s_i_current
        type: u1
      - id: charge_current
        type: u1
      - id: spare
        type: u1 
    instances:
      eps_mode_raw:
        value: eps_mode
      battery_voltage_v:
        value: battery_voltage * 20 /1000.0
      battery_temperature_c:
        value: battery_temperature
      min_battery_voltage_v:
        value: min_battery_voltage * 20 /1000.0
      max_battery_voltage_v:
        value: max_battery_voltage * 20 /1000.0
      avg_battery_voltage_v:
        value: avg_battery_voltage * 20 /1000.0
      avg_charge_current_ma:
        value: avg_charge_current * 12.0
      max_charge_current_ma:
        value:  max_charge_current * 12.0
      z_minu_face_temperature_c:
        value: z_minu_face_temperature
      o_b_d_h_current_ma:
        value: o_b_d_h_current
      e_p_s_current_ma:
        value: e_p_s_current
      t_t_c_micro_c_current_ma:
        value: t_t_c_micro_c_current
      t_t_c_p_a_current_ma:
        value: t_t_c_p_a_current
      d_o_s_i_current_ma:
        value: d_o_s_i_current * 5.0
      charge_current_ma:
        value: charge_current * 12.0
        
        
  ttc:
    seq:
      - id: mode 
        type: u1
      - id: number_of_ttc_resets 
        type: u2
      - id: last_reset_cause 
        type: u1
      - id: number_of_received_valid_packets 
        type: u2
      - id: number_of_transmitted_packets 
        type: u2
      - id: measured_transmission_power 
        type: u2
      - id: last_error_code 
        type: u1
      - id: power_configuration 
        type: u1
      - id: power_amplifier_temperature 
        type: s1
      - id: rssi_of_last_received_packet 
        type: u1
      - id: frequency_deviation_of_last_received_packet 
        type: u1
      - id: beacon_period 
        type: u1 
    instances:
      mode_raw: 
        value: mode
      number_of_ttc_resets_raw:
        value: number_of_ttc_resets
      last_reset_cause_raw:
        value: last_reset_cause
      number_of_received_valid_packets_raw:
        value: number_of_received_valid_packets 
      number_of_transmitted_packets_raw: 
        value: number_of_transmitted_packets
      measured_transmission_power_raw:
        value: measured_transmission_power
      last_error_code_raw:
        value: last_error_code
      power_configuration_raw:
        value: power_configuration
      power_amplifier_temperature_c: 
        value: power_amplifier_temperature
      rssi_of_last_received_packet_dbm:
        value: rssi_of_last_received_packet * -1
      frequency_deviation_of_last_received_packet_hz:
        value: frequency_deviation_of_last_received_packet * 17
      beacon_period_s:
        value: beacon_period
        

  ham_message:
    seq:
      - id: last_message_rssi
        type: u1
      - id: message
        size: 133
    instances:
      last_message_rssi_raw:
        value: last_message_rssi
      ham_message_raw:
        value: message        
