meta:
  id: spino
  endian: le

seq:
  - id: ax25_frame
    type: ax25_frame
    doc-ref: 'https://www.tapr.org/pub_ax25.html'

types:
  ax25_frame:
    seq:
    - id: ax25_header
      type: ax25_header
    - id: contenttype
      type: 
        switch-on: ax25_header.src_ssid 
        cases:
           0x03: digi
           0x02: payload
           0x00: payload
           0x05: payload


  ax25_header:
    seq:
      - id: dest_callsign_raw
        type: dest_callsign_raw
      - id: dest_ssid_raw
        type: u1
      - id: src_callsign_raw
        type: src_callsign_raw
      - id: src_ssid_raw
        type: u1
      - id: ctl
        type: u1
      - id: pid
        type: u1
    instances:
      src_ssid:
        value: (src_ssid_raw & 0x0f) >> 1
      dest_ssid:
        value: (dest_ssid_raw & 0x0f) >> 1
  dest_callsign_raw:
    seq:
      - id: dest_callsign_ror
        process: ror(1)
        size: 6
        type: dest_callsign
  src_callsign_raw:
    seq:
      - id: src_callsign_ror
        process: ror(1)
        type: src_callsign
        size: 6
  dest_callsign:
    seq:
      - id: dest_callsign
        type: str
        encoding: ASCII
        size: 6
  src_callsign:
    seq:
      - id: src_callsign
        type: str
        encoding: ASCII
        size: 6
  digi:
   seq:
      - id: size
        type: u2
      - id: data
        encoding: ASCII
        type: str
        size-eos: true
 

        
        
  payload:
    seq:
      - id: size
        type: u2
      - id: cmd_id
        type: u2
      - id: response_type
        type: u1
      - id: error_code
        type: u1
      - id: spare
        type: u2
      - id: timestamps
        type: u8
      - id: responsetype
        type: 
          switch-on: response_type 
          cases:
#           0x00: telemetry
           0x80: telemetry
           0x10: response_beacon_expe
           0x40: response_typeab
           0x41: information_message
           32: response_typeab
        if: error_code == 0
        
  response_beacon_expe: 
    seq:
      - id: size
        type: u2
      - id: id 
        type: u4
      - id: expereceived 
        type: u4
      - id: expereceivedfail
        type: u4      
      - id: expe_delay
        type: u2
      - id: idf
        type: u2

        
          
  information_message:
    seq:
      - id: resp_type
        type: u2
      - id: spare 
        type: u2
      - id: index
        type: u1
      - id: used
        type: u1 
      - id:  messagea
        encoding: ASCII
        type: str
        size-eos: true
             

  response_typeab:
    seq: 

      - id: response_data
        type: 
          switch-on: _parent.cmd_id 
          cases:
           0x00: telemetry
           0x23: list_mail_box
           0x25: mailbox_message
           0x26: mailbox_message
           0x49: tle
           0x67: configuration
           0x65: response_setdatavalue
           0x66: response_getdatavalue
           76: list_mail_box
           0x78: response_last_drop
           0x79: response_all_drop
           122: response_last_drop
           0x82: response_last_log
           0x86: response_last_log
           0x83: response_all_log
           #0x11: s_frame          
       # if: payload.error_code == 0     
       
           
  telemetry:
    seq:
      - id: size
        type: u2 
      - id: nb_commande_received
        type: u4
      - id: nb_commande_with_error
        type: u4
      - id: nb_frame_not_processed
        type: u4
      - id: nb_digipeater_messsage_processed
        type: u4
      - id: nb_experiemntal_command
        type: u4
      - id: nb_experiemntal_command_error
        type: u4
      - id: nb_mailbox_commande_received
        type: u4
      - id: nb_mailbox_error_commande_received
        type: u4
      - id: nb_payload_commande_received
        type: u4
      - id: nb_payload_command_with_error
        type: u4
      - id: timestamps
        type: u8
      - id: last_reset_cause
        type: u2
      - id: current_state
        type: u2
      - id: spare
        type: u4
      - id:  crc
        type: u2
        
  mailbox_message:
    seq:

      - id: timetamps
        type: u8
      - id: callsign
        size: 6
      - id: index 
        type: u1
      - id: value
        size-eos: true 
        
  response_last_drop:
    seq:
      - id: timetamps
        type: u8
      - id: size_msg
        type: u2
      - id:  crc
        type: u2
        
  response_all_drop:
    seq:
      - id:  message
        size-eos: true 
        
  list_mail_box:
    seq:
      - id: indi
        size: 6
      - id: nbmsage
        size: 1
      - id: indi2
        size: 6
      - id: nbmsage2
        size: 1
      - id: value
        size-eos: true 
        
        
  response_all_log:
    seq:
      - id:  message
        size-eos: true 

  tle:
    seq:
      - id:  messagea
        encoding: ASCII
        type: str
        size-eos: true    
        
  response_last_log:
    seq:

      - id: timetamps
        type: u8
      - id: priority
        type: u1
      - id:  messagea
        encoding: ASCII
        type: str
        size-eos: true
        
  response_setdatavalue:
    seq:
      - id: field_id
        type: u1
      - id:  crc
        type: u2


  response_getdatavalue:
    seq:
      - id: size
        type: u2 
      - id: field_id
        type: u1
      - id: field_size
        type: u1
      - id: value_data
        type:
          switch-on: field_id 
          cases:
           0x80: version
           0x01: spino_delay
           0x02: callsign    
           0x03: callsign  
           0x04: callsign  
      - id: value_data2
        size-eos: true  
        
  version:
    seq: 
      - id : version
        type: u2
        
        
  spino_delay:
    seq: 
      - id : spino_delay
        type: u1     
      - id:  crc
        type: u2
  callsign:
    seq: 
      - id : callsign
        type: str
        encoding: ASCII
        size: 6
      - id:  crc
        type: u2
  configuration:
    seq:

      - id: spino_tx_frequency
        type: u4
      - id: spino_rx_frequency
        type: u4
      - id: spino_tx_modem_speed
        type: u2
      - id: spino_rx_modem_speed
        type: u2
      - id: spino_rx_modem_mode
        type: u1
      - id: spino_tx_modem_mode
        type: u1      
      - id: telemetry_delay
        type: u1
      - id: multimode
        type: u1
      - id: info_message_actif
        type: u1
      - id: delay_info_message
        type: u1        
      - id: src_callsign
        type: str
        encoding: ASCII
        size: 6
      - id: dest_callsign
        type: str
        encoding: ASCII
        size: 6
      - id:  payload_callsign
        type: str
        encoding: ASCII
        size: 6
      - id:  crc
        type: u2
#      - id: config_data
#        size-eos: true  
        
